//
//  MHVisualTheme.h
//  SQLEditor
//
//  Created by Angus Hardie on 15/10/2019.
//  Copyright © 2019 MalcolmHardie Solutions Limited. All rights reserved.
//

#import <Cocoa/Cocoa.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSInteger, MHVisualThemeType) {
        MHVisualThemeTypeDefault,
        MHVisualThemeTypeLight,
        MHVisualThemeTypeDark,
};
    
    

@interface MHVisualTheme : NSObject <NSCoding>

+ (MHVisualTheme*)visualTheme;
+ (MHVisualTheme*)visualThemeOfType:(MHVisualThemeType)type;


@property (retain) NSMutableDictionary* colorDictionary;
@property (copy) NSString* fileName;


- (BOOL)loadStoredData;
- (void)saveFileData;
- (void)resetColors;



+ (MHVisualTheme*)lightTheme;
+ (MHVisualTheme*)darkTheme;

- (void)setDefaultColors;
- (void)setColor:(NSColor*)color forKey:(NSString*)key;
- (NSColor*)colorForKey:(NSString*)key;

// schema
- (NSColor*)schemaBackgroundColor;
- (NSColor*)gridColor;
- (NSColor*)pageLineColor;

// objects
- (NSColor*)frameColor;
- (NSColor*)backgroundColor;
- (NSColor*)textColor;
- (NSColor*)objectIconColor;
- (NSColor*)objectIconColorReversed;


- (NSColor*)objectHandleColor;
- (NSColor*)objectHandleColorBackground;


// (from prefs in older releases)
- (NSColor*)tableSelectedColor;
- (NSColor*)tableNotSelectedColor;
- (NSColor*)tableBackgroundColor;

- (NSColor*)commentSelectedColor;
- (NSColor*)commentNotSelectedColor;
- (NSColor*)commentBackgroundColor;



// connectors



- (NSColor*)connectorHandleColorSelected;
- (NSColor*)connectorHandleColor;

- (NSColor*)labelTextColor;



- (NSColor*)connectorIsSource;
- (NSColor*)connectorIsDest;
- (NSColor*)connectorDefault;


@end

NS_ASSUME_NONNULL_END
