/* JDBCDriverManager */

#import <Cocoa/Cocoa.h>

/**
 Manages Database (JDBC) drivers
 **/

@interface JDBCDriverManager : NSObject <NSTableViewDataSource,NSTableViewDelegate>
{
    IBOutlet id addButton;
    IBOutlet id deleteButton;
    IBOutlet NSTableView* driverListTable;
    IBOutlet id driverNameField;
	IBOutlet id driverUrlField;
    IBOutlet id driverProtocolField;
	
	NSMutableArray* driverList; // 
	NSMutableArray* combinedDriverList;
	
	NSMutableArray* discoveredDriverList;
    
    NSTimer* discoveryTimeoutTimer;
    
    id _javaWorkerThread;
	
}


@property (retain) NSTimer* discoveryTimeoutTimer;


+ (JDBCDriverManager*)sharedJDBCDriverManager;

//- (IBAction)addDriver:(id)sender;
- (IBAction)deleteDriver:(id)sender;
- (BOOL)writeDriversToFile;
- (void)reloadData;
- (NSUInteger)driverCount;
- (id)getDriverAtIndex:(NSUInteger)index;

- (NSUInteger)numberOfItemsInComboBox:(NSComboBox *)aComboBox;
- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(NSUInteger)index;

- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(NSUInteger)rowIndex;
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)column	row:(NSInteger)rowIndex;
- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)column
			  row:(NSInteger)rowIndex;
- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView;


+ (NSString*)driverListFileLocation;
- (BOOL)readDataFromPath:(NSString*) path;
- (NSString*)stringForObjectAtIndex:(NSUInteger)index;

- (NSString*)subProtocolAtIndex:(NSInteger)index;
- (NSString*)urlAtIndex:(NSUInteger)index;
- (NSString*)nameAtIndex:(NSUInteger)index;


- (int)indexForDriverWithUrl:(NSString*)url;
- (void)addDriverFromDictionary:(id)driverDictionary;
- (void)discoverDrivers;
- (void)combineDriverLists;

- (BOOL)isSelectedDriverAutoDiscovered;
-(void)deleteDriverAtIndex:(NSUInteger)index;

- (NSArray*)combinedDriverList;

+ (void)addDriverToList:(id)driverDictionary;
@end
